--------------------------------------------------------------------------------
-- Title   : type library
-- Project : TimIQ
--------------------------------------------------------------------------------
-- Description : type declaration lib
--------------------------------------------------------------------------------
-- File     : ADC_lib.vhdl
-- Revision : x.x.x
-- Created  : August 30, 2016
-- Updated  : November 22, 2016
--------------------------------------------------------------------------------
-- Author       : Jean-Paul Ricaud
-- Organization : Synchrotron Soleil
-- Web          : http://www.synchrotron-soleil.fr - https://www.maxiv.lu.se
-- Email        : jean-paul.ricaud@synchrotron-soleil.fr
--------------------------------------------------------------------------------
-- Copyright (C) 2015 - 2017 Synchrotron Soleil - MAX IV Laboratory
--
-- This program is free software: you can redistribute it and/or modify
-- it under the terms of the GNU Lesser General Public License as published by
-- the Free Software Foundation, either version 3 of the License, or
-- (at your option) any later version.
--
-- This program is distributed in the hope that it will be useful,
-- but WITHOUT ANY WARRANTY; without even the implied warranty of
-- MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
-- GNU Lesser General Public License for more details.
--
-- You should have received a copy of the GNU Lesser General Public License
-- along with this program.  If not, see <http://www.gnu.org/licenses/>
--------------------------------------------------------------------------------
--
--------------------------------------------------------------------------------
-- Modifications :
--
--------------------------------------------------------------------------------

library ieee;
use ieee.std_logic_1164.all;
use ieee.numeric_std.all;

--------------------------------------------------------------------------------
--------------------------------------------------------------------------------
--------------------------------------------------------------------------------
package ADC_lib is

  type ADC_program_register_array is array (0 to 2) of std_logic_vector (23 downto 0);

  constant c_ADC_reg_configuration : ADC_program_register_array := (
    X"000000", -- manuel channel selection
    X"000B01", -- input range channel 0 ; +/- 1.25 x Vref
    X"000D01"  -- input range channel 1 ; +/- 1.25 x Vref
  );

--  type ADC_command_register_array is array (0 to 2) of std_logic_vector (33 downto 0);
  type ADC_command_register_array is array (0 to 1) of std_logic_vector (33 downto 0);

  constant c_ADC_reg_CH0 : ADC_command_register_array := (
    "1100000000000000000000000000000000", -- select channel 0 ; read data from channel 1 - old
    "1100010000000000000000000000000000"  -- select channel 1 ; read data from channel 0 - new
  );

  constant c_ADC_reg_CH1 : ADC_command_register_array := (
    "1100010000000000000000000000000000", -- select channel 1 ; read data from channel 0 - old
    "1100000000000000000000000000000000"  -- select channel 0 ; read data from channel 1 - new
  );

end package ADC_lib;

--------------------------------------------------------------------------------
--------------------------------------------------------------------------------
--------------------------------------------------------------------------------
package body ADC_lib is

end package body ADC_lib;

