--------------------------------------------------------------------------------
-- Title   : BBB
-- Project : TimIQ
--------------------------------------------------------------------------------
-- Description : BeagleBone interface
--------------------------------------------------------------------------------
-- File     : BBB_top.vhd
-- Revision : x.x.x
-- Created  : August 8, 2016
-- Updated  : November 13, 2016
--------------------------------------------------------------------------------
-- Author       : Jean-Paul Ricaud
-- Organization : Synchrotron Soleil
-- Web          : http://www.synchrotron-soleil.fr - https://www.maxiv.lu.se
-- Email        : jean-paul.ricaud@synchrotron-soleil.fr
--------------------------------------------------------------------------------
-- Copyright (C) 2015 - 2016 Synchrotron Soleil - MAX IV Laboratory
--
-- This program is free software: you can redistribute it and/or modify
-- it under the terms of the GNU Lesser General Public License as published by
-- the Free Software Foundation, either version 3 of the License, or
-- (at your option) any later version.
--
-- This program is distributed in the hope that it will be useful,
-- but WITHOUT ANY WARRANTY; without even the implied warranty of
-- MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
-- GNU Lesser General Public License for more details.
--
-- You should have received a copy of the GNU Lesser General Public License
-- along with this program.  If not, see <http://www.gnu.org/licenses/>
--------------------------------------------------------------------------------
--
--------------------------------------------------------------------------------
-- Modifications :
--
--------------------------------------------------------------------------------

library ieee;
use ieee.std_logic_1164.all;
use ieee.numeric_std.all;

library work;
use work.registers_type.register_array;

--------------------------------------------------------------------------------
--------------------------------- ENTITY ---------------------------------------
--------------------------------------------------------------------------------
entity BBB_top is
  port (
    p_reset             : in    std_logic; -- reset input
    p_clkin             : in    std_logic; -- clk input
    p_address           : out   std_logic_vector (6 downto 0);
    p_read_register     : out   std_logic_vector (23 downto 0); -- read register
    p_read_ready        : out   std_logic; -- data from master available
    p_write_register    : in    std_logic_vector (7 downto 0); -- write register
    p_write_request     : out   std_logic; -- ask data to be send to master
    p_write_data_ready  : in    std_logic; -- data to be send to master ready
    p_BBB_RSTn          : out   std_logic; -- BeagleBone reset
    p_BBB_I2C1_SCL      : in    std_logic; -- BeagleBone I2C 1 clock
    p_BBB_I2C1_SDA      : inout std_logic; -- BeagleBone I2C 1 data
    p_BBB_I2C2_SCL      : in    std_logic; -- BeagleBone I2C 2 clock
    p_BBB_I2C2_SDA      : inout std_logic; -- BeagleBone I2C 2 data
    p_BBB_SPI1_CS0      : in    std_logic; -- BeagleBone SPI 1 CS
    p_BBB_SPI1_CLK      : in    std_logic; -- BeagleBone SPI 1 clock
    p_BBB_SPI1_D0       : inout std_logic; -- BeagleBone SPI 1 data 0
    p_BBB_SPI1_D1       : inout std_logic; -- BeagleBone SPI 1 data 1
    p_BBB_CLKOUT2       : in    std_logic; -- BeagleBone clock out 2
    p_BBB_UART1_TxD     : in    std_logic; -- BeagleBone UART 1 RxD
    p_BBB_UART1_RxD     : out   std_logic;  -- BeagleBone UART 1 TxD

    p_test              : out   std_logic_vector (4 downto 0)
  );
end entity BBB_top;

--------------------------------------------------------------------------------
------------------------------- ARCHITECTURE -----------------------------------
--------------------------------------------------------------------------------
architecture rtl_BBB_top of BBB_top is
  ------------------------------------------------------------------------------
  -- constant
  ------------------------------------------------------------------------------

  ------------------------------------------------------------------------------
  -- signal
  ------------------------------------------------------------------------------

  ------------------------------------------------------------------------------
  -- Attributes
  ------------------------------------------------------------------------------

  ------------------------------------------------------------------------------
  --------------------------------- Main ---------------------------------------
  ------------------------------------------------------------------------------
  begin

  ------------------------------------------------------------------------------
  -- I2C2 acccess
  ------------------------------------------------------------------------------
  -- I2C read write
  -- Slave mode ; 7 bits address ; 2 back-to-back 8 bits = 16 bits data
  I2C_access : entity work.BBB_I2C (rtl_BBB_I2C)
    port map (
      p_clkin             => p_clkin,
      p_reset             => p_reset,
      p_address           => p_address,
      p_read_register     => p_read_register,
      p_read_strobe       => p_read_ready,
      p_write_register    => p_write_register,
      p_write_request     => p_write_request,
      p_write_data_ready  => p_write_data_ready,
      p_SCL               => p_BBB_I2C2_SCL,
      p_SDA               => p_BBB_I2C2_SDA,

      p_test              => p_test
    );

  ------------------------------------------------------------------------------
  ------------------------------------------------------------------------------

  p_BBB_RSTn      <= '1'; -- BBB reset
  p_BBB_I2C1_SDA  <= 'Z'; -- BeagleBone I2C 1 data
  p_BBB_SPI1_D0   <= 'Z'; -- BeagleBone SPI 1 data 0
  p_BBB_SPI1_D1   <= 'Z'; -- BeagleBone SPI 1 data 1
  p_BBB_UART1_RxD <= 'Z'; -- BeagleBone UART 1 TxD

end architecture rtl_BBB_top;
