--------------------------------------------------------------------------------
-- Title   : lsfr
-- Project : TimIQ
--------------------------------------------------------------------------------
-- Description : clock division module
--------------------------------------------------------------------------------
-- File     : clk_lsfr.vhd
-- Revision : x.x.x
-- Created  : August 3, 2016
-- Updated  : November 14, 2017
--------------------------------------------------------------------------------
-- Author       : Jean-Paul Ricaud
-- Organization : Synchrotron Soleil
-- Web          : http://www.synchrotron-soleil.fr - https://www.maxiv.lu.se
-- Email        : jean-paul.ricaud@synchrotron-soleil.fr
--------------------------------------------------------------------------------
-- Copyright (C) 2015 - 2016 Synchrotron Soleil - MAX IV Laboratory
--
-- This program is free software: you can redistribute it and/or modify
-- it under the terms of the GNU Lesser General Public License as published by
-- the Free Software Foundation, either version 3 of the License, or
-- (at your option) any later version.
--
-- This program is distributed in the hope that it will be useful,
-- but WITHOUT ANY WARRANTY; without even the implied warranty of
-- MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
-- GNU Lesser General Public License for more details.
--
-- You should have received a copy of the GNU Lesser General Public License
-- along with this program.  If not, see <http://www.gnu.org/licenses/>
--------------------------------------------------------------------------------
--
--------------------------------------------------------------------------------
-- Modifications :
--
--------------------------------------------------------------------------------

library ieee;
use ieee.std_logic_1164.all;
use ieee.numeric_std.all;

library work;
use work.clk_lib.all;

--------------------------------------------------------------------------------
--------------------------------- ENTITY ---------------------------------------
--------------------------------------------------------------------------------
entity clk_lsfr is
  generic (
    g_lfsr : integer;
    g_polynome : lfsr_polynome
  );
  port (
    p_clkin  : in  std_logic; -- input clock
    p_clkout : out std_logic  -- output clock
  );
end entity clk_lsfr;

--------------------------------------------------------------------------------
------------------------------- ARCHITECTURE -----------------------------------
--------------------------------------------------------------------------------
architecture rtl_clk_lsfr of clk_lsfr is
  ------------------------------------------------------------------------------
  -- constant
  ------------------------------------------------------------------------------
  constant c_size : integer := g_polynome(g_polynome'LEFT);

  ------------------------------------------------------------------------------
  -- signal
  ------------------------------------------------------------------------------
  signal s_lfsr       : std_logic_vector (c_size downto 0) := (others => '0');
  signal s_lfsrlsb    : std_logic := '0';
  signal s_clk        : std_logic := '0';
  signal s_xnor_chain : std_logic_vector (g_polynome'length downto 0) := (others => '1');


  ------------------------------------------------------------------------------
  -- Attributes
  ------------------------------------------------------------------------------

  ------------------------------------------------------------------------------
  --------------------------------- Main ---------------------------------------
  ------------------------------------------------------------------------------
  begin

  ------------------------------------------------------------------------------
  shifter : process (p_clkin)
  begin
    if (rising_edge(p_clkin)) then
      if(s_lfsr = std_logic_vector(to_unsigned(g_lfsr, c_size + 1))) then
        s_lfsr <= (OTHERS => '0');
        s_clk  <= '1';
      else
        s_lfsr <= s_lfsr((c_size - 1) downto 0) & s_lfsrlsb;
        s_clk  <= '0';
      end if;
    end if;
  end process;

  reduc_xnor : for i in 1 to g_polynome'length generate
  begin
    s_xnor_chain(i) <= s_xnor_chain(i - 1) xnor s_lfsr(g_polynome(i - 1));
  end generate;

  ------------------------------------------------------------------------------
  ------------------------------------------------------------------------------
  s_lfsrlsb <= s_xnor_chain(g_polynome'length);
  p_clkout  <= s_clk;

end architecture rtl_clk_lsfr;
