--------------------------------------------------------------------------------
-- Title   : registers definition
-- Project : TimIQ
--------------------------------------------------------------------------------
-- Description : definition of the registers of the board
--------------------------------------------------------------------------------
-- File     : Hermes_registers.vhdl
-- Revision : x.x.x
-- Created  : November 04, 2016
-- Updated  : December 09, 2016
--------------------------------------------------------------------------------
-- Author       : Jean-Paul Ricaud
-- Organization : Synchrotron Soleil
-- Web          : http://www.synchrotron-soleil.fr - https://www.maxiv.lu.se
-- Email        : jean-paul.ricaud@synchrotron-soleil.fr
--------------------------------------------------------------------------------
-- Copyright (C) 2015 - 2017 Synchrotron Soleil - MAX IV Laboratory
--
-- This program is free software: you can redistribute it and/or modify
-- it under the terms of the GNU Lesser General Public License as published by
-- the Free Software Foundation, either version 3 of the License, or
-- (at your option) any later version.
--
-- This program is distributed in the hope that it will be useful,
-- but WITHOUT ANY WARRANTY; without even the implied warranty of
-- MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
-- GNU Lesser General Public License for more details.
--
-- You should have received a copy of the GNU Lesser General Public License
-- along with this program.  If not, see <http://www.gnu.org/licenses/>
--------------------------------------------------------------------------------
--
--------------------------------------------------------------------------------
-- Modifications :
--
--------------------------------------------------------------------------------

library ieee;
use ieee.std_logic_1164.all;
use ieee.numeric_std.all;

--------------------------------------------------------------------------------
--------------------------------------------------------------------------------
--------------------------------------------------------------------------------
package Hermes_registers is

  ------------------------------------------------------------------------------
  -- Registers
  ------------------------------------------------------------------------------
  -- Write registers

  -- Read registers
  constant c_board_id     : std_logic_vector (31 downto 0) := X"5AC0FA5A"; -- board ID for TimIQ
  constant c_firmware_rev : std_logic_vector (31 downto 0) := X"00000064"; -- firmware's version

  -- I2C addresses
  constant c_add_command  : std_logic_vector (6 downto 0) := "0010000"; -- command register ; 0x10
  constant c_add_I        : std_logic_vector (6 downto 0) := "0010001"; -- set I register ; 0x11
  constant c_add_Q        : std_logic_vector (6 downto 0) := "0010010"; -- set Q register ; 0x12
  constant c_add_Temperatue_setup : std_logic_vector (6 downto 0) := "0010011"; -- set temperature register ; 0x13

  constant c_add_Vs_0     : std_logic_vector (6 downto 0) := "0010100"; -- Vs(7 downto 0)   register ; 0x14
  constant c_add_Vs_1     : std_logic_vector (6 downto 0) := "0010101"; -- Vs(15 downto 8)  register ; 0x15
  constant c_add_Vs_2     : std_logic_vector (6 downto 0) := "0010110"; -- Vs(23 downto 16) register ; 0x16
  constant c_add_Vc_0     : std_logic_vector (6 downto 0) := "0010111"; -- Vc(7 downto 0)   register ; 0x17
  constant c_add_Vc_1     : std_logic_vector (6 downto 0) := "0011000"; -- Vc(15 downto 8)  register ; 0x18
  constant c_add_Vc_2     : std_logic_vector (6 downto 0) := "0011001"; -- Vc(23 downto 16) register ; 0x19
  constant c_add_Temp_0   : std_logic_vector (6 downto 0) := "0011010"; -- T(7 downto 0)    register ; 0x1A
  constant c_add_Temp_1   : std_logic_vector (6 downto 0) := "0011011"; -- T(15 downto 8)   register ; 0x1B
  constant c_add_Temp_2   : std_logic_vector (6 downto 0) := "0011100"; -- T(23 downto 16)  register ; 0x1C
  constant c_add_Temp_3   : std_logic_vector (6 downto 0) := "0011101"; -- T(31 downto 24)  register ; 0x1D

end package Hermes_registers;

--------------------------------------------------------------------------------
--------------------------------------------------------------------------------
--------------------------------------------------------------------------------
package body Hermes_registers is

end package body Hermes_registers;
