--------------------------------------------------------------------------------
-- Title   : type library
-- Project : TimIQ
--------------------------------------------------------------------------------
-- Description : type declaration lib
--------------------------------------------------------------------------------
-- File     : PLL_lib.vhdl
-- Revision : x.x.x
-- Created  : August 9, 2016
-- Updated  : March 02, 2018
--------------------------------------------------------------------------------
-- Author       : Jean-Paul Ricaud
-- Organization : Synchrotron Soleil
-- Web          : http://www.synchrotron-soleil.fr - https://www.maxiv.lu.se
-- Email        : jean-paul.ricaud@synchrotron-soleil.fr
--------------------------------------------------------------------------------
-- Copyright (C) 2015 - 2018 Synchrotron Soleil - MAX IV Laboratory
--
-- This program is free software: you can redistribute it and/or modify
-- it under the terms of the GNU Lesser General Public License as published by
-- the Free Software Foundation, either version 3 of the License, or
-- (at your option) any later version.
--
-- This program is distributed in the hope that it will be useful,
-- but WITHOUT ANY WARRANTY; without even the implied warranty of
-- MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
-- GNU Lesser General Public License for more details.
--
-- You should have received a copy of the GNU Lesser General Public License
-- along with this program.  If not, see <http://www.gnu.org/licenses/>
--------------------------------------------------------------------------------
--
--------------------------------------------------------------------------------
-- Modifications :
--
--------------------------------------------------------------------------------

library ieee;
use ieee.std_logic_1164.all;
use ieee.numeric_std.all;

--------------------------------------------------------------------------------
--------------------------------------------------------------------------------
--------------------------------------------------------------------------------
package PLL_lib is

  type PLL_register_array is array (0 to 11) of std_logic_vector (31 downto 0);

  constant c_PLL_reg_configuration : PLL_register_array := (
    X"00000017", -- R7
--    X"0000008D", -- R13
    X"000000CD", -- R13
    X"0000001C", -- R12
    X"28001409", -- R9
--    X"0111CF88", -- R8 ; fixed frequency temperature compensation 24
    X"0111CE48", -- R8 ; frequency temperature compensation 4
--    X"001F3366", -- R6 ; Enout enabled
    X"001FBB76", -- R6 ; Enout enabled
    X"A0000005", -- R5
    X"12001FF4", -- R4 ; OSCin freq for 255 MHz and higer ; VCO_DIV = bypass mode ; R3 = 200R ; R4 = 200R ; C3 = 5pF ; C4 = 5pF
    X"00387F03", -- R3 ; full mode ; charge pump 32x ; digital lock detect ; negative CPP ; doubler = 1 ; 3 order filter ; DITh weak
    X"04000052", -- R2
    X"000000B1", -- R1 ; R = 11
    X"00000580"  -- R0 ; N = 88
  );

  constant c_PLL_reg_calibration : std_logic_vector (31 downto 0) := X"00000580"; -- R0 ; N = 88

end package PLL_lib;

--------------------------------------------------------------------------------
--------------------------------------------------------------------------------
--------------------------------------------------------------------------------
package body PLL_lib is

end package body PLL_lib;

