--------------------------------------------------------------------------------
-- Title   : I2C read testbench
-- Project : TimIQ
--------------------------------------------------------------------------------
-- Description : I2C read ; test bench
--               Read from FPGA
--------------------------------------------------------------------------------
-- File     : BBB_top_I2C_read_tb.vhd
-- Revision : x.x.x
-- Created  : September 16, 2016
-- Updated  : November 02, 2016
--------------------------------------------------------------------------------
-- Author       : Jean-Paul Ricaud
-- Organization : Synchrotron Soleil
-- Web          : http://www.synchrotron-soleil.fr
-- Email        : jean-paul.ricaud@synchrotron-soleil.fr
--------------------------------------------------------------------------------
-- Copyright (C) 2015 - 2016 Synchrotron Soleil - MAX IV Laboratory
--
-- This program is free software: you can redistribute it and/or modify
-- it under the terms of the GNU Lesser General Public License as published by
-- the Free Software Foundation, either version 3 of the License, or
-- (at your option) any later version.
--
-- This program is distributed in the hope that it will be useful,
-- but WITHOUT ANY WARRANTY; without even the implied warranty of
-- MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
-- GNU Lesser General Public License for more details.
--
-- You should have received a copy of the GNU Lesser General Public License
-- along with this program.  If not, see <http://www.gnu.org/licenses/>
--------------------------------------------------------------------------------
--
--------------------------------------------------------------------------------
-- Modifications :
--
--------------------------------------------------------------------------------

library ieee;
use ieee.std_logic_1164.all;
use ieee.numeric_std.all;

--------------------------------------------------------------------------------
--------------------------------- ENTITY ---------------------------------------
--------------------------------------------------------------------------------
entity BBB_top_I2C_read_tb is
end entity BBB_top_I2C_read_tb;

--------------------------------------------------------------------------------
------------------------------- ARCHITECTURE -----------------------------------
--------------------------------------------------------------------------------
architecture archi_BBB_top_I2C_read_tb of BBB_top_I2C_read_tb is

  -- constants
  constant Period_10MHz  : TIME := 100.000 ps; -- divide by 1000
  constant Period_100kHz : TIME := 100.000 ns; -- divide by 1000

  -- signals
  signal s_rst_sr           : std_logic := '0';
  signal s_clk20MHz         : std_logic := '0';
  signal s_read_register    : std_logic_vector (15 downto 0) := (others => '0');
  signal s_address          : std_logic_vector (6 downto 0)  := (others => '0');
  signal s_read_ready       : std_logic := '0';
  signal s_write_register   : std_logic_vector (15 downto 0) := (others => '0');
  signal s_write_strobe     : std_logic := '0';
  signal s_write_data_ready : std_logic := '0';
  signal s_ack              : std_logic := '1';

  signal s_RSTn      : std_logic := '0';
  signal s_I2C1_SCL  : std_logic := '1';
  signal s_I2C1_SDA  : std_logic := '1';
  signal s_I2C2_SCL  : std_logic := '1';
  signal s_I2C2_SDA  : std_logic := '1';
  signal s_SPI1_CS0  : std_logic := '0';
  signal s_SPI1_CLK  : std_logic := '0';
  signal s_SPI1_D0   : std_logic := '0';
  signal s_SPI1_D1   : std_logic := '0';
  signal s_CLKOUT2   : std_logic := '0';
  signal s_UART1_TxD : std_logic := '0';
  signal s_UART1_RxD : std_logic := '0';


  ----------------------------------------------------------------------------
  ----------------------------------------------------------------------------
  begin

  DUT : entity work.BBB_top (rtl_BBB_top)
  port map (
    p_reset            => s_rst_sr, -- reset in
    p_clkin            => s_clk20MHz,
    p_read_register    => s_read_register,
    p_address          => s_address,
    p_read_ready       => s_read_ready,
    p_write_register   => s_write_register,
    p_write_strobe     => s_write_strobe,
    p_write_data_ready => s_write_data_ready,
    p_BBB_RSTn         => s_RSTn,
    p_BBB_I2C1_SCL     => s_I2C1_SCL,
    p_BBB_I2C1_SDA     => s_I2C1_SDA,
    p_BBB_I2C2_SCL     => s_I2C2_SCL,
    p_BBB_I2C2_SDA     => s_I2C2_SDA,
    p_BBB_SPI1_CS0     => s_SPI1_CS0,
    p_BBB_SPI1_CLK     => s_SPI1_CLK,
    p_BBB_SPI1_D0      => s_SPI1_D0,
    p_BBB_SPI1_D1      => s_SPI1_D1,
    p_BBB_CLKOUT2      => s_CLKOUT2,
    p_BBB_UART1_TxD    => s_UART1_TxD,
    p_BBB_UART1_RxD    => s_UART1_RxD
  );

  -- Clock generator
  process
  begin
    s_clk20MHz <= '0';
    wait for (Period_10MHz / 4);
    s_clk20MHz <= '1';
    wait for (Period_10MHz / 4);
  end process;

  process
  begin
    -- Init states
    s_rst_sr <= '0';
    wait for 1013 ps;
    s_rst_sr <= '1';
    wait for 525 ps;
    s_rst_sr <= '0';
    wait for 1024 ps;

    -- Start condition
    s_I2C2_SDA <= '0';
    wait for (Period_100kHz / 2);
    s_I2C2_SCL <= '0';
    wait for (Period_100kHz / 2);

    -- Address 6
    s_I2C2_SDA <= '0';
    s_I2C2_SCL <= '1';
    wait for (Period_100kHz / 2);
    s_I2C2_SDA <= '0';
    s_I2C2_SCL <= '0';
    wait for (Period_100kHz / 2);
    -- Address 5
    s_I2C2_SCL <= '1';
    wait for (Period_100kHz / 2);
    s_I2C2_SCL <= '0';
    wait for (Period_100kHz / 2);
    -- Address 4
    s_I2C2_SDA <= '1';
    s_I2C2_SCL <= '1';
    wait for (Period_100kHz / 2);
    s_I2C2_SDA <= '0';
    s_I2C2_SCL <= '0';
    wait for (Period_100kHz / 2);
    -- Address 3
    s_I2C2_SCL <= '1';
    wait for (Period_100kHz / 2);
    s_I2C2_SCL <= '0';
    wait for (Period_100kHz / 2);
    -- Address 2
    s_I2C2_SCL <= '1';
    wait for (Period_100kHz / 2);
    s_I2C2_SCL <= '0';
    wait for (Period_100kHz / 2);
    -- Address 1
    s_I2C2_SCL <= '1';
    wait for (Period_100kHz / 2);
    s_I2C2_SCL <= '0';
    wait for (Period_100kHz / 2);
    -- Address 0
    s_I2C2_SCL <= '1';
    wait for (Period_100kHz / 2);
    s_I2C2_SCL <= '0';
    s_I2C2_SDA <= '1';
    wait for (Period_100kHz / 2);

    -- Command write
    s_I2C2_SCL <= '1';
    wait for (Period_100kHz / 2);
    s_I2C2_SCL <= '0';
    wait for (Period_100kHz / 2);

    -- Get ack
    s_I2C2_SCL <= '1';
    s_ack <= s_I2C2_SDA;
    wait for (Period_100kHz / 2);
    s_I2C2_SCL <= '0';
    wait for (Period_100kHz / 2);

    -- Set data
    s_write_register <= X"5555";
    wait for (Period_10MHz / 2);
    s_write_data_ready <= '1';
    wait for (Period_10MHz / 2);
    s_write_data_ready <= '0';

    -- Write bit 15
    wait for (Period_10MHz);
    s_I2C2_SCL <= '1';
    wait for (Period_100kHz / 2);
    s_I2C2_SCL <= '0';
    wait for (Period_100kHz / 2);
    -- Write bit 14
    wait for (Period_10MHz);
    s_I2C2_SCL <= '1';
    wait for (Period_100kHz / 2);
    s_I2C2_SCL <= '0';
    wait for (Period_100kHz / 2);
    -- Write bit 13
    wait for (Period_10MHz);
    s_I2C2_SCL <= '1';
    wait for (Period_100kHz / 2);
    s_I2C2_SCL <= '0';
    wait for (Period_100kHz / 2);
    -- Write bit 12
    wait for (Period_10MHz);
    s_I2C2_SCL <= '1';
    wait for (Period_100kHz / 2);
    s_I2C2_SCL <= '0';
    wait for (Period_100kHz / 2);
    -- Write bit 11
    wait for (Period_10MHz);
    s_I2C2_SCL <= '1';
    wait for (Period_100kHz / 2);
    s_I2C2_SCL <= '0';
    wait for (Period_100kHz / 2);
    -- Write bit 10
    wait for (Period_10MHz);
    s_I2C2_SCL <= '1';
    wait for (Period_100kHz / 2);
    s_I2C2_SCL <= '0';
    wait for (Period_100kHz / 2);
    -- Write bit 9
    wait for (Period_10MHz);
    s_I2C2_SCL <= '1';
    wait for (Period_100kHz / 2);
    s_I2C2_SCL <= '0';
    wait for (Period_100kHz / 2);
    -- Write bit 8
    wait for (Period_10MHz);
    s_I2C2_SCL <= '1';
    wait for (Period_100kHz / 2);
    s_I2C2_SCL <= '0';
    wait for (Period_100kHz / 2);

    -- Get ack
    s_I2C2_SCL <= '1';
    s_I2C2_SDA <= '0';
    wait for (Period_100kHz / 2);
    s_I2C2_SCL <= '0';
    wait for (Period_100kHz / 2);

    -- Set data
    s_write_register <= X"AAAA";
    wait for (Period_10MHz / 2);
    s_write_data_ready <= '1';
    wait for (Period_10MHz / 2);
    s_write_data_ready <= '0';

    -- Write bit 7
    wait for (Period_10MHz);
    s_I2C2_SCL <= '1';
    wait for (Period_100kHz / 2);
    s_I2C2_SCL <= '0';
    wait for (Period_100kHz / 2);
    -- Write bit 6
    wait for (Period_10MHz);
    s_I2C2_SCL <= '1';
    wait for (Period_100kHz / 2);
    s_I2C2_SCL <= '0';
    wait for (Period_100kHz / 2);
    -- Write bit 5
    wait for (Period_10MHz);
    s_I2C2_SCL <= '1';
    wait for (Period_100kHz / 2);
    s_I2C2_SCL <= '0';
    wait for (Period_100kHz / 2);
    -- Write bit 4
    wait for (Period_10MHz);
    s_I2C2_SCL <= '1';
    wait for (Period_100kHz / 2);
    s_I2C2_SCL <= '0';
    wait for (Period_100kHz / 2);
    -- Write bit 3
    wait for (Period_10MHz);
    s_I2C2_SCL <= '1';
    wait for (Period_100kHz / 2);
    s_I2C2_SCL <= '0';
    wait for (Period_100kHz / 2);
    -- Write bit 2
    wait for (Period_10MHz);
    s_I2C2_SCL <= '1';
    wait for (Period_100kHz / 2);
    s_I2C2_SCL <= '0';
    wait for (Period_100kHz / 2);
    -- Write bit 1
    wait for (Period_10MHz);
    s_I2C2_SCL <= '1';
    wait for (Period_100kHz / 2);
    s_I2C2_SCL <= '0';
    wait for (Period_100kHz / 2);
    -- Write bit 0
    wait for (Period_10MHz);
    s_I2C2_SCL <= '1';
    wait for (Period_100kHz / 2);
    s_I2C2_SCL <= '0';
    wait for (Period_100kHz / 2);

    -- Get ack
    s_I2C2_SCL <= '1';
    s_I2C2_SDA <= '0';
    wait for (Period_100kHz / 2);
    s_I2C2_SCL <= '0';
    wait for (Period_100kHz / 2);

    -- Stop condition
    s_I2C2_SDA <= '0';
    wait for (Period_100kHz / 2);
    s_I2C2_SCL <= '1';
    wait for (Period_100kHz / 2);
    s_I2C2_SDA <= '1';


    -- *** END ***
    wait;
  end process;

  -----------------------------------------------------------------------------
  -----------------------------------------------------------------------------

end architecture archi_BBB_top_I2C_read_tb;
