--------------------------------------------------------------------------------
-- Title   : ADC SPI testbench
-- Project : TimIQ
--------------------------------------------------------------------------------
-- Description : ADC SPI transmission ; test bench
--------------------------------------------------------------------------------
-- File     : ADC_tb.vhd
-- Revision : x.x.x
-- Created  : August 31, 2016
-- Updated  : August 31, 2016
--------------------------------------------------------------------------------
-- Author       : Jean-Paul Ricaud
-- Organization : Synchrotron Soleil
-- Web          : http://www.synchrotron-soleil.fr
-- Email        : jean-paul.ricaud@synchrotron-soleil.fr
--------------------------------------------------------------------------------
-- Copyright (C) 2015 - 2016 Synchrotron Soleil - MAX IV Laboratory
--
-- This program is free software: you can redistribute it and/or modify
-- it under the terms of the GNU Lesser General Public License as published by
-- the Free Software Foundation, either version 3 of the License, or
-- (at your option) any later version.
--
-- This program is distributed in the hope that it will be useful,
-- but WITHOUT ANY WARRANTY; without even the implied warranty of
-- MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
-- GNU Lesser General Public License for more details.
--
-- You should have received a copy of the GNU Lesser General Public License
-- along with this program.  If not, see <http://www.gnu.org/licenses/>
--------------------------------------------------------------------------------
--
--------------------------------------------------------------------------------
-- Modifications :
--
--------------------------------------------------------------------------------

library ieee;
use ieee.std_logic_1164.all;
use ieee.numeric_std.all;

--------------------------------------------------------------------------------
--------------------------------- ENTITY ---------------------------------------
--------------------------------------------------------------------------------
entity ADC_top_tb is
end entity ADC_top_tb;

--------------------------------------------------------------------------------
------------------------------- ARCHITECTURE -----------------------------------
--------------------------------------------------------------------------------
architecture archi_ADC_top_tb of ADC_top_tb is

  -- constants
  constant Period_10MHz  : TIME := 50.000 ps; -- divide by 1000

  -- signals
  signal s_rst_sr     : std_logic := '0';
  signal s_clk20MHz   : std_logic := '0';

  signal s_get_data   : std_logic := '0';
  signal s_data_ready : std_logic := '0';
  signal s_ADC_CSn    : std_logic := '0';
  signal s_ADC_SCK    : std_logic := '0';
  signal s_ADC_MOSI   : std_logic := '0';
  signal s_ADC_MISO   : std_logic := '0';
  signal s_ADC_ALARM  : std_logic := '0';
  signal s_ADC_ENA    : std_logic := '0';
  signal s_ADC_ENB    : std_logic := '0';
  signal s_ADC_ENC    : std_logic := '0';
  signal s_ADC_END    : std_logic := '0';

  ----------------------------------------------------------------------------
  ----------------------------------------------------------------------------
  begin

  DUT : entity work.ADC_top (rtl_ADC_top)
  port map (
    p_reset      => s_rst_sr, -- reset in
    p_clkin      => s_clk20MHz,
    p_get_data   => s_get_data,
    p_data_ready => s_data_ready,
    p_ADC_CSn    => s_ADC_CSn,
    p_ADC_SCK    => s_ADC_SCK,
    p_ADC_MOSI   => s_ADC_MOSI,
    p_ADC_MISO   => s_ADC_MISO,
    p_ADC_ALARM  => s_ADC_ALARM,
    p_ADC_ENA    => s_ADC_ENA,
    p_ADC_ENB    => s_ADC_ENB,
    p_ADC_ENC    => s_ADC_ENC,
    p_ADC_END    => s_ADC_END
  );

  -- Clock generator
  process
  begin
    s_clk20MHz <= '0';
    wait for (Period_10MHz / 2);
    s_clk20MHz <= '1';
    wait for (Period_10MHz / 2);
  end process;

  process
  begin
    -- Init states
    s_ADC_MISO <= '1';
    s_rst_sr <= '0';
    wait for 1000 ps;
    s_rst_sr <= '1';
    wait for 500 ps;
    s_rst_sr <= '0';
    wait for 10200 ps;
    s_rst_sr <= '1';
    wait for 500 ps;
    s_rst_sr <= '0';
    wait for 10500 ps;
    s_get_data <= '1';
    wait for 500 ps;
    s_get_data <= '0';
    wait for (Period_10MHz * 64);
    s_ADC_MISO <= '0';
    wait for (Period_10MHz * 32);
    s_ADC_MISO <= '1';
    wait for (Period_10MHz * 32);
    s_ADC_MISO <= '0';
    wait for (Period_10MHz * 32);
    s_ADC_MISO <= '1';

    -- *** END ***
    wait;
  end process;

  -----------------------------------------------------------------------------
  -----------------------------------------------------------------------------

end architecture archi_ADC_top_tb;
