--------------------------------------------------------------------------------
-- Title   : DAC SPI testbench
-- Project : TimIQ
--------------------------------------------------------------------------------
-- Description : DAC SPI transmission ; test bench
--------------------------------------------------------------------------------
-- File     : DAC_tb.vhd
-- Revision : x.x.x
-- Created  : August 22, 2016
-- Updated  : August 22, 2016
--------------------------------------------------------------------------------
-- Author       : Jean-Paul Ricaud
-- Organization : Synchrotron Soleil
-- Web          : http://www.synchrotron-soleil.fr
-- Email        : jean-paul.ricaud@synchrotron-soleil.fr
--------------------------------------------------------------------------------
-- Copyright (C) 2015 - 2016 Synchrotron Soleil - MAX IV Laboratory
--
-- This program is free software: you can redistribute it and/or modify
-- it under the terms of the GNU Lesser General Public License as published by
-- the Free Software Foundation, either version 3 of the License, or
-- (at your option) any later version.
--
-- This program is distributed in the hope that it will be useful,
-- but WITHOUT ANY WARRANTY; without even the implied warranty of
-- MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
-- GNU Lesser General Public License for more details.
--
-- You should have received a copy of the GNU Lesser General Public License
-- along with this program.  If not, see <http://www.gnu.org/licenses/>
--------------------------------------------------------------------------------
--
--------------------------------------------------------------------------------
-- Modifications :
--
--------------------------------------------------------------------------------

library ieee;
use ieee.std_logic_1164.all;
use ieee.numeric_std.all;

--------------------------------------------------------------------------------
--------------------------------- ENTITY ---------------------------------------
--------------------------------------------------------------------------------
entity DAC_top_tb is
end entity DAC_top_tb;

--------------------------------------------------------------------------------
------------------------------- ARCHITECTURE -----------------------------------
--------------------------------------------------------------------------------
architecture archi_DAC_top_tb of DAC_top_tb is

  -- constants
  constant Period_10MHz  : TIME := 50.000 ps; -- divide by 1000

  -- signals
  signal s_rst_sr   : std_logic;
  signal s_clk20MHz : std_logic;

  signal s_DAC_CSn  : std_logic := '0';
  signal s_DAC_SCK  : std_logic := '0';
  signal s_DAC_MOSI : std_logic := '0';
  signal s_DAC_MISO : std_logic := '0';
  signal s_DAC_ENA  : std_logic := '0';
  signal s_DAC_ENB  : std_logic := '0';

  ----------------------------------------------------------------------------
  ----------------------------------------------------------------------------
  begin

  DUT : entity work.DAC_top (rtl_DAC_top)
  port map (
    p_reset    => s_rst_sr, -- reset in
    p_clkin    => s_clk20MHz,
    p_DAC_CSn  => s_DAC_CSn,
    p_DAC_SCK  => s_DAC_SCK,
    p_DAC_MOSI => s_DAC_MOSI,
    p_DAC_MISO => s_DAC_MISO,
    p_DAC_ENA  => s_DAC_ENA,
    p_DAC_ENB  => s_DAC_ENB
  );

  -- Clock generator
  process
  begin
    s_clk20MHz <= '0';
    wait for (Period_10MHz / 2);
    s_clk20MHz <= '1';
    wait for (Period_10MHz / 2);
  end process;

  process
  begin
    -- Init states
    s_rst_sr <= '0';
    wait for 1000 ps;
    s_rst_sr <= '1';
    wait for 500 ps;
    s_rst_sr <= '0';
    wait for 10200 ps;
    s_rst_sr <= '1';
    wait for 500 ps;
    s_rst_sr <= '0';

    -- *** END ***
    wait;
  end process;

  -----------------------------------------------------------------------------
  -----------------------------------------------------------------------------

end architecture archi_DAC_top_tb;
