--------------------------------------------------------------------------------
-- Title   : PLL SPI testbench
-- Project : TimIQ
--------------------------------------------------------------------------------
-- Description : PLL SPI transmission ; test bench
--------------------------------------------------------------------------------
-- File     : PLL_tb.vhd
-- Revision : x.x.x
-- Created  : August 3, 2016
-- Updated  : February 26, 2018
--------------------------------------------------------------------------------
-- Author       : Jean-Paul Ricaud
-- Organization : Synchrotron Soleil
-- Web          : http://www.synchrotron-soleil.fr
-- Email        : jean-paul.ricaud@synchrotron-soleil.fr
--------------------------------------------------------------------------------
-- Copyright (C) 2015 - 2018 Synchrotron Soleil - MAX IV Laboratory
--
-- This program is free software: you can redistribute it and/or modify
-- it under the terms of the GNU Lesser General Public License as published by
-- the Free Software Foundation, either version 3 of the License, or
-- (at your option) any later version.
--
-- This program is distributed in the hope that it will be useful,
-- but WITHOUT ANY WARRANTY; without even the implied warranty of
-- MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
-- GNU Lesser General Public License for more details.
--
-- You should have received a copy of the GNU Lesser General Public License
-- along with this program.  If not, see <http://www.gnu.org/licenses/>
--------------------------------------------------------------------------------
--
--------------------------------------------------------------------------------
-- Modifications :
--
--------------------------------------------------------------------------------

library ieee;
use ieee.std_logic_1164.all;
use ieee.numeric_std.all;

--------------------------------------------------------------------------------
--------------------------------- ENTITY ---------------------------------------
--------------------------------------------------------------------------------
entity PLL_top_tb is
end entity PLL_top_tb;

--------------------------------------------------------------------------------
------------------------------- ARCHITECTURE -----------------------------------
--------------------------------------------------------------------------------
architecture archi_PLL_top_tb of PLL_top_tb is

  -- constants
  constant Period_20MHz  : TIME := 50.000 ps; -- divide by 1000
  constant Period_847kHz : TIME := 1180.000 ps;

  -- signals
  signal s_rst_sr   : std_logic;
  signal s_clk20MHz : std_logic;

  signal s_laser_trigger : std_logic := '0';
  signal s_PLL_LE        : std_logic := '0';
  signal s_PLL_CLK       : std_logic := '0';
  signal s_PLL_DATA      : std_logic := '0';
  signal s_PLL_CE        : std_logic := '0';
  signal s_PLL_Enout     : std_logic := '0';
  signal s_PLL_LOCKED    : std_logic := '0';
  signal s_PLL_ENA       : std_logic := '0';
  signal s_PLL_ENB       : std_logic := '0';
  signal s_PLL_ENC       : std_logic := '0';
  signal s_PLL_END       : std_logic := '0';

  ----------------------------------------------------------------------------
  ----------------------------------------------------------------------------
  begin

  DUT : entity work.PLL_top (rtl_PLL_top)
  port map (
    p_clkin         => s_clk20MHz,
    p_reset         => s_rst_sr, -- reset in
    p_laser_trigger => s_laser_trigger,
    p_PLL_LE        => s_PLL_LE,
    p_PLL_CE        => s_PLL_CE,
    p_PLL_CLK       => s_PLL_CLK,
    p_PLL_DATA      => s_PLL_DATA,
    p_PLL_ENout     => s_PLL_ENout,
    p_PLL_LOCKED    => s_PLL_LOCKED,
    p_PLL_ENA       => s_PLL_ENA,
    p_PLL_ENB       => s_PLL_ENB,
    p_PLL_ENC       => s_PLL_ENC,
    p_PLL_END       => s_PLL_END
  );

  -- Clock generator
  process
  begin
    s_clk20MHz <= '0';
    wait for (Period_20MHz / 2);
    s_clk20MHz <= '1';
    wait for (Period_20MHz / 2);
  end process;

  -- Clock generator
  process
  begin
    s_laser_trigger <= '0';
    wait for (Period_847kHz / 2);
    s_laser_trigger <= '1';
    wait for (Period_847kHz / 2);
  end process;

  process
  begin
    -- Init states
    s_rst_sr <= '0';
    wait for 1000 ps;
    s_rst_sr <= '1';
    wait for 500 ps;
    s_rst_sr <= '0';
    wait for 10200 ps;
    s_rst_sr <= '1';
    wait for 500 ps;
    s_rst_sr <= '0';

    -- *** END ***
    wait;
  end process;

  -----------------------------------------------------------------------------
  -----------------------------------------------------------------------------

end architecture archi_PLL_top_tb;
