--------------------------------------------------------------------------------
-- Title   : Reset testbench
-- Project : TimIQ
--------------------------------------------------------------------------------
-- Description : reset ; test bench
--------------------------------------------------------------------------------
-- File     : reset_tb.vhd
-- Revision : x.x.x
-- Created  : August 22, 2016
-- Updated  : August 22, 2016
--------------------------------------------------------------------------------
-- Author       : Jean-Paul Ricaud
-- Organization : Synchrotron Soleil
-- Web          : http://www.synchrotron-soleil.fr
-- Email        : jean-paul.ricaud@synchrotron-soleil.fr
--------------------------------------------------------------------------------
-- Copyright (C) 2015 - 2016 Synchrotron Soleil - MAX IV Laboratory
--
-- This program is free software: you can redistribute it and/or modify
-- it under the terms of the GNU Lesser General Public License as published by
-- the Free Software Foundation, either version 3 of the License, or
-- (at your option) any later version.
--
-- This program is distributed in the hope that it will be useful,
-- but WITHOUT ANY WARRANTY; without even the implied warranty of
-- MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
-- GNU Lesser General Public License for more details.
--
-- You should have received a copy of the GNU Lesser General Public License
-- along with this program.  If not, see <http://www.gnu.org/licenses/>
--------------------------------------------------------------------------------
--
--------------------------------------------------------------------------------
-- Modifications :
--
--------------------------------------------------------------------------------

library ieee;
use ieee.std_logic_1164.all;
use ieee.numeric_std.all;

--------------------------------------------------------------------------------
--------------------------------- ENTITY ---------------------------------------
--------------------------------------------------------------------------------
entity reset_tb is
end entity reset_tb;

--------------------------------------------------------------------------------
------------------------------- ARCHITECTURE -----------------------------------
--------------------------------------------------------------------------------
architecture archi_reset_tb of reset_tb is

  -- constants
  constant Period_10MHz : TIME := 10.000 ps; -- divide by 1000

  -- signals
  signal s_clk20MHz  : std_logic;
  signal s_clk500mHz : std_logic;

  signal s_cmdReg       : std_logic := '0';
  signal s_CSW          : std_logic := '0';
  signal s_PWRON_config : std_logic := '0';
  signal s_fpga_RST     : std_logic := '0';
  signal s_board_RST    : std_logic := '0';

  ----------------------------------------------------------------------------
  ----------------------------------------------------------------------------
  begin

  DUT : entity work.reset (rtl_reset)
  port map (
    p_clkin        => s_clk20MHz,
    p_clk500mHz    => s_clk500mHz,
    p_cmd_REG0     => s_cmdReg,
    p_CSW          => s_CSW,
    p_PWRON_config => s_PWRON_config,
    p_fpga_RST     => s_fpga_RST,
    p_board_RST    => s_board_RST
  );

  -- Clock generator
  process
  begin
    s_clk20MHz <= '0';
    wait for (Period_10MHz / 2);
    s_clk20MHz <= '1';
    wait for (Period_10MHz / 2);
  end process;

  process
  begin
    s_clk500mHz <= '0';
    wait for (Period_10MHz * 100);
    s_clk500mHz <= '1';
    wait for (Period_10MHz * 100);
  end process;

  -----------------------------------------------------------------------------
  -----------------------------------------------------------------------------

end architecture archi_reset_tb;
