--------------------------------------------------------------------------------
-- Title   : Top
-- Project : TimIQ
--------------------------------------------------------------------------------
-- Description : Top sheet
--------------------------------------------------------------------------------
-- File     : top.vhd
-- Revision : x.x.x
-- Created  : August 02, 2016
-- Updated  : March 06, 2018
--------------------------------------------------------------------------------
-- Author       : Jean-Paul Ricaud
-- Organization : Synchrotron Soleil - MAX IV Laboratory
-- Web          : http://www.synchrotron-soleil.fr - https://www.maxiv.lu.se
-- Email        : jean-paul.ricaud@synchrotron-soleil.fr
--------------------------------------------------------------------------------
-- Copyright (C) 2015 - 2018 Synchrotron Soleil - MAX IV Laboratory
--
-- This program is free software: you can redistribute it and/or modify
-- it under the terms of the GNU Lesser General Public License as published by
-- the Free Software Foundation, either version 3 of the License, or
-- (at your option) any later version.
--
-- This program is distributed in the hope that it will be useful,
-- but WITHOUT ANY WARRANTY; without even the implied warranty of
-- MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
-- GNU Lesser General Public License for more details.
--
-- You should have received a copy of the GNU Lesser General Public License
-- along with this program. If not, see <http://www.gnu.org/licenses/>
--------------------------------------------------------------------------------
--
--------------------------------------------------------------------------------
-- Modifications :
--   Version 1.0.0 ; August 2, 2016 ; Jean-Paul Ricaud
--     * First draft
--
--------------------------------------------------------------------------------

library ieee;
use ieee.std_logic_1164.all;
use ieee.numeric_std.all;

Library unisim;
use unisim.vcomponents.all;

library work;
use work.registers_type.all;

--------------------------------------------------------------------------------
--------------------------------- ENTITY ---------------------------------------
--------------------------------------------------------------------------------
entity top is
  port (
    -- Ethernet PHY interface
    pin_PHY_TXD    : out   std_logic_vector (7 downto 0);
    pin_PHY_TXEN   : out   std_logic;
    pin_PHY_TXCLK  : in    std_logic;
    pin_PHY_GTXCLK : out   std_logic;
    pin_PHY_TXER   : out   std_logic;
    pin_PHY_RXD    : in    std_logic_vector (7 downto 0);
    pin_PHY_RXDV   : in    std_logic;
    pin_PHY_RXER   : in    std_logic;
    pin_PHY_RXCLK  : in    std_logic;
    pin_PHY_COL    : in    std_logic;
    pin_PHY_CRS    : in    std_logic;
    pin_PHY_MDC    : out   std_logic;
    pin_PHY_MDIO   : inout std_logic;
    pin_PHY_INT    : in    std_logic;
    pin_PHY_HPD    : out   std_logic;
    pin_PHY_RSTn   : out   std_logic;

    -- Clock
    pin_CLK_100MHz_p : in    std_logic; -- 100 MHz clock +
    pin_CLK_100MHz_n : in    std_logic; -- 100 MHz clock -

    -- Peltier controleor (RS485)
    pin_PCTRL_DE   : out    std_logic; -- Peltier controlor RS485 DE
    pin_PCTRL_D    : out    std_logic; -- Peltier controlor RS485 D
    pin_PCTRL_REn  : out    std_logic; -- Peltier controlor RS485 /RE
    pin_PCTRL_R    : in     std_logic; -- Peltier controlor RS485 R

    -- PLL
    pin_PLL_LE     : out    std_logic; -- PLL LE
    pin_PLL_CE     : out    std_logic; -- PLL CE
    pin_PLL_CLK    : out    std_logic; -- PLL clock
    pin_PLL_DATA   : out    std_logic; -- PLL data
    pin_PLL_ENout  : out    std_logic; -- PLL ENout
    pin_PLL_LOCKED : in     std_logic; -- PLL locked
    pin_PLL_ENA    : out    std_logic; -- PLL buffer enable A
    pin_PLL_ENB    : out    std_logic; -- PLL buffer enable B
    pin_PLL_ENC    : out    std_logic; -- PLL buffer enable C
    pin_PLL_END    : out    std_logic; -- PLL buffer enable D

    -- ADC
    pin_ADC_RSTn  : out    std_logic; -- ADC reset
    pin_ADC_CSn   : out    std_logic; -- ADC chip select
    pin_ADC_SCK   : out    std_logic; -- ADC clock
    pin_ADC_MOSI  : out    std_logic; -- ADC Master Out Slave In
    pin_ADC_MISO  : in     std_logic; -- ADC Master In Slave Out
    pin_ADC_ALARM : in     std_logic; -- ADC alarm
    pin_ADC_ENA   : out    std_logic; -- ADC buffer enable A
    pin_ADC_ENB   : out    std_logic; -- ADC buffer enable B
    pin_ADC_ENC   : out    std_logic; -- ADC buffer enable C (noted as DAC_ENC in schematics)
    pin_ADC_END   : out    std_logic; -- ADC buffer enable D (noted as DAC_END in schematics)

    -- DAC
    pin_DAC_CSn  : out    std_logic; -- DAC chip select
    pin_DAC_SCK  : out    std_logic; -- DAC clock
    pin_DAC_MOSI : out    std_logic; -- DAC Master Out Slave In
    pin_DAC_MISO : in     std_logic; -- DAC Master In Slave Out
    pin_DAC_ENA  : out    std_logic; -- DAC buffer enable A
    pin_DAC_ENB  : out    std_logic; -- DAC buffer enable B

    -- Frequency dividers
    pin_FD0_N0   : out    std_logic; -- frequency divider
    pin_FD0_N1   : out    std_logic; -- frequency divider
    pin_FD0_BYP  : out    std_logic; -- frequency divider
    pin_FD1_N0   : out    std_logic; -- frequency divider
    pin_FD1_N1   : out    std_logic; -- frequency divider
    pin_FD1_BYP  : out    std_logic; -- frequency divider
    pin_FD2_BYP  : out    std_logic; -- frequency divider
    pin_FD2_DIV1 : out    std_logic; -- frequency divider
    pin_FD_ENA   : out    std_logic; -- frequency divider buffer enable A
    pin_FD_ENB   : out    std_logic; -- frequency divider buffer enable B
    pin_FD_ENC   : out    std_logic; -- frequency divider buffer enable C
    pin_FD_END   : out    std_logic; -- frequency divider buffer enable D

    -- BeagleBone bus
    pin_BBB_RSTn      : out   std_logic; -- BeagleBone reset
    pin_BBB_I2C1_SCL  : in    std_logic; -- BeagleBone I2C 1 clock
    pin_BBB_I2C1_SDA  : inout std_logic; -- BeagleBone I2C 1 data
    pin_BBB_I2C2_SCL  : in    std_logic; -- BeagleBone I2C 2 clock
    pin_BBB_I2C2_SDA  : inout std_logic; -- BeagleBone I2C 2 data
    pin_BBB_SPI1_CS0  : in    std_logic; -- BeagleBone SPI 1 CS
    pin_BBB_SPI1_CLK  : in    std_logic; -- BeagleBone SPI 1 clock
    pin_BBB_SPI1_D0   : inout std_logic; -- BeagleBone SPI 1 data 0
    pin_BBB_SPI1_D1   : inout std_logic; -- BeagleBone SPI 1 data 1
    pin_BBB_CLKOUT2   : in    std_logic; -- BeagleBone clock out 2
    pin_BBB_UART1_TxD : in    std_logic; -- BeagleBone UART 1 RxD
    pin_BBB_UART1_RxD : out   std_logic; -- BeagleBone UART 1 TxD

    -- IO signals
    pin_inTTL     : in    std_logic_vector (4 downto 0); -- TTL inputs
    pin_outTTL    : out   std_logic_vector (4 downto 0); -- TTL outputs

    -- Digital interface spares signals
    pin_SPARE_out : out   std_logic_vector (3 downto 0); -- spare signals between the CPU and the RD boards
    pin_SPARE_in  : in    std_logic_vector (7 downto 4); -- spare signals between the CPU and the RD boards

    -- Board reset
    pin_RST_BRDn  : out   std_logic; -- Board reset output
    -- Front Panel LEDs
    pin_LED       : out   std_logic_vector (1 downto 0); -- LEDs ; 0 = red, 1 = green
    -- Extension connector
    pin_EXTS      : out   std_logic_vector (7 downto 0); -- Test header
    -- Switch connector
    pin_SW        : in    std_logic_vector (5 downto 0)  -- Configuration dip switch
  );
end entity top;

--------------------------------------------------------------------------------
------------------------------- ARCHITECTURE -----------------------------------
--------------------------------------------------------------------------------
architecture rtl_top of top is
  ------------------------------------------------------------------------------
  -- component
  ------------------------------------------------------------------------------
  -- Chipscope
  component chipscope_icon_v1_06_a_0
  port (
    control0 : inout std_logic_vector(35 downto 0)
  );
  end component;

  component chipscope_ila_v1_05_a_0
  port (
    control : inout std_logic_vector(35 downto 0);
    clk     : in    std_logic;
    data    : in    std_logic_vector(31 downto 0);
    trig0   : in    std_logic_vector(0 to 0)
  );
  end component;

  ------------------------------------------------------------------------------
  -- constant
  ------------------------------------------------------------------------------

  ------------------------------------------------------------------------------
  -- signal
  ------------------------------------------------------------------------------
  -- Chipscope
  signal s_chipscope_control : std_logic_vector(35 downto 0) := (others => '0');
  signal s_chipscope_data    : std_logic_vector(31 downto 0) := (others => '0');
  signal s_chipscope_trig    : std_logic_vector(0 to 0) := (others => '0');

  -- Registers ; write from BBB/CPU to internal registers
  signal s_config_register : std_logic_vector(23 downto 0) := (others => '0');
  signal s_command_reg     : std_logic_vector(23 downto 0) := (others => '0');
  signal s_DAC_reg         : std_logic_vector(31 downto 0) := (others => '0');
  signal s_Temperatue_setup_reg : std_logic_vector(23 downto 0) := (others => '0');

  -- Registers ; read from internal registers to BBB/CPU
  signal s_ADC_values     : std_logic_vector (33 downto 0);
  signal s_Temperatue_reg : std_logic_vector(31 downto 0) := (others => '0');

  -- Latch the data from the BBB and send to a component
  -- 00000001 | commande register
  -- 00000010 | DAC register
  -- 00000100 | set temperature register
  signal s_latch_data   : std_logic_vector (7 downto 0) := (others => '0');
  -- Request a data from a component
  -- 00000001 | ADC
  -- 00000010 | temperature
  signal s_request_data : std_logic_vector (7 downto 0) := (others => '0');
  -- Data from a component ready to be sent to the BBB
  -- 00000001 | ADC
  -- 00000010 | temperature
  signal s_data_ready       : std_logic_vector (7 downto 0) := (others => '0');
  signal s_write_data_ready : std_logic := '0';  -- data ready for the BBB
                                                 -- OR'ed between all bits of s_data_ready

  -- FPGA clock
  signal s_clk100MHz : std_logic := '0'; -- 100 MHz clock
  signal s_clk20MHz  : std_logic := '0'; -- 10 MHz clock
  signal s_clk500mHz : std_logic := '0'; -- 0.5 mHz clock

  -- Resets
  signal s_reset        : std_logic := '0';
  signal s_PWRON_config : std_logic := '0';
  signal s_LED_RST      : std_logic := '0';

  -- BBB
  signal s_BBB_read_reg      : std_logic_vector (23 downto 0) := (others => '0');
  signal s_BBB_write_reg     : std_logic_vector (7 downto 0) := (others => '0');
  signal s_BBB_address       : std_logic_vector (6 downto 0) := (others => '0');
  signal s_BBB_read_ready    : std_logic; -- CPU had sent a data
  signal s_BBB_write_request : std_logic; -- CPU requests a data

  -- PLL
  signal s_PLL_LOCKED : std_logic := '0';
  signal s_PLL_sync   : std_logic := '1';

  -- Test
  signal s_test_signal : std_logic_vector (7 downto 0) := (others => '0');

  ------------------------------------------------------------------------------
  -- Attributes
  ------------------------------------------------------------------------------
  attribute PERIOD : string;
  attribute PERIOD of s_clk100MHz : signal is "10 ns";

  attribute BUFFER_TYPE : string;
  attribute BUFFER_TYPE of s_clk100MHz : signal is "BUFG";

  ------------------------------------------------------------------------------
  --------------------------------- Main ---------------------------------------
  ------------------------------------------------------------------------------
  begin

  -- IBUFGDS: Differential Global Clock Input Buffer
  --          Spartan-6
  -- Xilinx HDL Language Template, version 14.7
  IBUFGDS_inst : IBUFGDS
  generic map (
    DIFF_TERM    => FALSE,   -- Differential Termination
    IBUF_LOW_PWR => TRUE,    -- Low power (TRUE) vs. performance (FALSE) setting for referenced I/O standards
    IOSTANDARD   => "DEFAULT")
  port map (
    O  => s_clk100MHz,      -- Clock buffer output
    I  => pin_CLK_100MHz_p, -- Diff_p clock buffer input (connect directly to top-level port)
    IB => pin_CLK_100MHz_n  -- Diff_n clock buffer input (connect directly to top-level port)
  );

  -- Chipscope
  chipscope_icon : chipscope_icon_v1_06_a_0
  port map (
    control0 => s_chipscope_control
  );

  chipscope_ila : chipscope_ila_v1_05_a_0
  port map (
    control => s_chipscope_control,
    clk     => s_clk20MHz,
    data    => s_chipscope_data,
    trig0   => s_chipscope_trig
  );

  ------------------------------------------------------------------------------
  ------------------------------------------------------------------------------

  -- Clock generator
  clk_gen : entity work.clk_top (rtl_clk_top)
    port map (
      p_clk100MHz => s_clk100MHz, -- clock 100 MHz
      p_clk20MHz  => s_clk20MHz,  -- clock 20 MHz
      p_reset     => s_reset,     -- reset in
      p_clk500mHz => s_clk500mHz  -- clock 0.5 Hz
    );

  -- Reset block
  reset : entity work.reset (rtl_reset)
    port map (
      p_clkin        => s_clk20MHz,
      p_clk500mHz    => s_clk500mHz,      -- clock 0.5 Hz
      p_cmd_REG0     => s_command_reg(0), -- command register bit 0 ; reset
      p_CSW          => s_latch_data(0),  -- chip select of internal write transfert
      p_PWRON_config => s_PWRON_config,   -- power on configuration signal
      p_fpga_RST     => s_reset,          -- reset out
      p_LED_RST      => s_LED_RST,        -- light the red led before starting config
      p_board_RST    => pin_RST_BRDn      -- board reset out
    );

  -- PLL
  PLL : entity work.PLL_top (rtl_PLL_top)
    port map (
      p_reset         => s_PWRON_config,     -- power on configuration signal
      p_clkin         => s_clk20MHz,
      p_cmd_REG1      => s_command_reg(1),   -- command register bit 1 ; PLL recalibrate
      p_CSW           => s_latch_data(0),    -- chip select of internal write transfert
      p_SW            => pin_SW(5 downto 0), -- inhibit PLL in MAX IV configuration
      p_PLL_LE        => pin_PLL_LE,
      p_PLL_CE        => pin_PLL_CE,
      p_PLL_CLK       => pin_PLL_CLK,
      p_PLL_DATA      => pin_PLL_DATA,
      p_PLL_ENout     => pin_PLL_ENout,
      p_PLL_ENA       => pin_PLL_ENA,
      p_PLL_ENB       => pin_PLL_ENB,
      p_PLL_ENC       => pin_PLL_ENC,
      p_PLL_END       => pin_PLL_END,
      p_PLL_sync      => s_PLL_sync,
      p_laser_trigger => pin_inTTL(0)
    );

  -- DAC
  DAC : entity work.DAC_top (rtl_DAC_top)
    port map (
      p_reset        => s_PWRON_config, -- power on configuration signal
      p_clkin        => s_clk20MHz,
      p_DAC_reg      => s_DAC_reg,
      p_write_strobe => s_latch_data(1),
      p_DAC_CSn      => pin_DAC_CSn,
      p_DAC_SCK      => pin_DAC_SCK,
      p_DAC_MOSI     => pin_DAC_MOSI,
      p_DAC_MISO     => pin_DAC_MISO,
      p_DAC_ENA      => pin_DAC_ENA,
      p_DAC_ENB      => pin_DAC_ENB
    );

  -- ADC
  ADC : entity work.ADC_top (rtl_ADC_top)
    port map (
      --p_reset      => s_reset, -- reset in
      p_reset      => s_PWRON_config, -- power on configuration signal
      p_clkin      => s_clk20MHz,
      p_get_data   => s_request_data(1 downto 0),
      p_ADC_datas  => s_ADC_values,
      p_data_ready => s_data_ready(0),
      p_ADC_RSTn   => pin_ADC_RSTn,
      p_ADC_CSn    => pin_ADC_CSn,
      p_ADC_SCK    => pin_ADC_SCK,
      p_ADC_MOSI   => pin_ADC_MOSI,
      p_ADC_MISO   => pin_ADC_MISO,
      p_ADC_ALARM  => pin_ADC_ALARM,
      p_ADC_ENA    => pin_ADC_ENA,
      p_ADC_ENB    => pin_ADC_ENB,
      p_ADC_ENC    => pin_ADC_ENC,
      p_ADC_END    => pin_ADC_END

--      p_test       => s_test_signal(5 downto 1)
    );

  -- Frequency divider
  FreqDivider : entity work.FreqDivider_top (rtl_FreqDivider_top)
    port map (
      p_reset    => s_reset, -- reset in
      p_SW       => pin_SW(5 downto 0),  -- SOLEIL / MAX IV configuration
      p_FD0_N0   => pin_FD0_N0,
      p_FD0_N1   => pin_FD0_N1,
      p_FD0_BYP  => pin_FD0_BYP,
      p_FD1_N0   => pin_FD1_N0,
      p_FD1_N1   => pin_FD1_N1,
      p_FD1_BYP  => pin_FD1_BYP,
      p_FD2_BYP  => pin_FD2_BYP,
      p_FD2_DIV1 => pin_FD2_DIV1,
      p_FD_ENA   => pin_FD_ENA,
      p_FD_ENB   => pin_FD_ENB,
      p_FD_ENC   => pin_FD_ENC,
      p_FD_END   => pin_FD_END
    );

  -- IOs driver
  IOs : entity work.IOs_top (rtl_IOs_top)
    port map (
      p_reset  => s_reset,   -- reset in
      p_inTTL  => pin_inTTL,
      p_outTTL => pin_outTTL,
      p_sync   => s_PLL_sync
    );

  -- Peltier controler
  PeltierCTRL : entity work.PeltierCTRL_top (rtl_PeltierCTRL_top)
    port map (
      p_reset     => s_reset, -- reset in
      p_clkin     => s_clk20MHz,
      p_PCTRL_DE  => pin_PCTRL_DE,
      p_PCTRL_D   => pin_PCTRL_D,
      p_PCTRL_REn => pin_PCTRL_REn,
      p_PCTRL_R   => pin_PCTRL_R
    );

  -- BBB
  BBB : entity work.BBB_top (rtl_BBB_top)
    port map (
      p_reset            => s_reset, -- reset in
      p_clkin            => s_clk20MHz,
      p_address          => s_BBB_address,
      p_read_register    => s_BBB_read_reg,
      p_read_ready       => s_BBB_read_ready,    -- CPU had sent a data
      p_write_register   => s_BBB_write_reg,
      p_write_request    => s_BBB_write_request, -- CPU request a data
      p_write_data_ready => s_write_data_ready,  -- CPU can retrive a data
      p_BBB_RSTn         => pin_BBB_RSTn,
      p_BBB_I2C1_SCL     => pin_BBB_I2C1_SCL,
      p_BBB_I2C1_SDA     => pin_BBB_I2C1_SDA,
      p_BBB_I2C2_SCL     => pin_BBB_I2C2_SCL,
      p_BBB_I2C2_SDA     => pin_BBB_I2C2_SDA,
      p_BBB_SPI1_CS0     => pin_BBB_SPI1_CS0,
      p_BBB_SPI1_CLK     => pin_BBB_SPI1_CLK,
      p_BBB_SPI1_D0      => pin_BBB_SPI1_D0,
      p_BBB_SPI1_D1      => pin_BBB_SPI1_D1,
      p_BBB_CLKOUT2      => pin_BBB_CLKOUT2,
      p_BBB_UART1_TxD    => pin_BBB_UART1_TxD,
      p_BBB_UART1_RxD    => pin_BBB_UART1_RxD,

      p_test             => s_test_signal(5 downto 1)
    );

  -- Interface between the BBB and the components
  Hermes : entity work.Hermes_top (rtl_Hermes_top)
    port map (
      p_clkin                => s_clk100MHz,
      p_reset                => s_reset, -- reset in
      p_address              => s_BBB_address,
      p_data_from_BBB        => s_BBB_read_reg,
      p_read_ready           => s_BBB_read_ready,    -- CPU had sent a data
      p_write_request        => s_BBB_write_request, -- CPU requests a data
      p_data_to_BBB          => s_BBB_write_reg,
      p_write_data_ready     => s_write_data_ready,  -- data ready to be sent to the BBB
                                                     -- OR'ed between all bits of s_data_ready
      p_command_reg          => s_command_reg,
      p_DAC_reg              => s_DAC_reg,
      p_Temperatue_setup_reg => s_Temperatue_setup_reg,
      p_ADC_datas            => s_ADC_values,
      p_Temperatue_reg       => s_Temperatue_reg,
      p_latch_data           => s_latch_data,        -- latch a data from the BBB
      p_request_data         => s_request_data,      -- request a data for the BBB
      p_data_ready           => s_data_ready,        -- data ready to be latch for the BBB

      p_test(1 downto 0)     => s_test_signal(7 downto 6)
    );

  -- Ethernet PHY
  phy : entity work.PHY_top (rtl_PHY_top)
    port map (
      p_reset      => s_reset,     -- reset in
      p_clkin      => s_clk100MHz,
      p_PHY_TXD    => pin_PHY_TXD,
      p_PHY_TXEN   => pin_PHY_TXEN,
      p_PHY_TXCLK  => pin_PHY_TXCLK,
      p_PHY_GTXCLK => pin_PHY_GTXCLK,
      p_PHY_TXER   => pin_PHY_TXER,
      p_PHY_RXD    => pin_PHY_RXD,
      p_PHY_RXDV   => pin_PHY_RXDV,
      p_PHY_RXER   => pin_PHY_RXER,
      p_PHY_RXCLK  => pin_PHY_RXCLK,
      p_PHY_COL    => pin_PHY_COL,
      p_PHY_CRS    => pin_PHY_CRS,
      p_PHY_MDC    => pin_PHY_MDC,
      p_PHY_MDIO   => pin_PHY_MDIO,
      p_PHY_INT    => pin_PHY_INT,
      p_PHY_HPD    => pin_PHY_HPD,
      p_PHY_RSTn   => pin_PHY_RSTn
    );

  -- Front LEDs
  LEDs : entity work.LEDs (rtl_LEDs)
    port map (
      p_clkin        => s_clk20MHz,    -- clock 20MHz
      p_clk500mHz    => s_clk500mHz,   -- clock 0.5 Hz
      p_reset        => s_reset,       -- reset
      p_red_LED      => pin_LED(0),    -- red LED
      p_green_LED    => pin_LED(1),    -- green LED
      p_LED_RST      => s_LED_RST      -- power on configuration signal
    );

  ------------------------------------------------------------------------------
  ------------------------------------------------------------------------------
  -- Chipscope
--  s_chipscope_data    <= "00000000" & s_BBB_read_reg;
--  s_chipscope_trig(0) <= s_test_signal(6);
--  s_chipscope_trig(0) <= s_BBB_read_ready;
--  s_chipscope_data    <= s_DAC_reg;
--  s_chipscope_trig(0) <= s_latch_data(1);
  s_chipscope_data    <= "000000000000000000000000" & s_request_data;
  s_chipscope_trig(0) <= s_request_data(1);


  s_config_register <= "000000000000000000" & pin_SW;
  s_PLL_LOCKED      <= pin_PLL_LOCKED;

  pin_EXTS(0)   <= s_PLL_LOCKED;
  pin_EXTS(1)   <= s_test_signal(1);
  pin_EXTS(2)   <= s_test_signal(2);
  pin_EXTS(3)   <= s_test_signal(3);
  pin_EXTS(4)   <= s_test_signal(4);
  pin_EXTS(5)   <= s_test_signal(5);
  pin_EXTS(6)   <= s_test_signal(6);
  pin_EXTS(7)   <= s_test_signal(7);

  pin_SPARE_out <= (others => '0');

end architecture rtl_top;
